(*
 * <COPYRIGHT>
 * Copyright 2005 Gerd Stolpmann
 *
 * <GPL>
 * This file is part of UMLMON.
 *
 * UMLMON is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * UMLMON is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with UMLMON; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * </>
 *)

(* $Id: umlmon_types.ml 209 2005-10-23 13:46:46Z gerd $ *)


(**********************************************************************)
(* Configuration types                                                *)
(**********************************************************************)

exception Lock_failure of string
  (* Raised when a lock fails. The string describes the resource that was
   * tried to be locked.
   *)

type jailfile =
    Jailfile of string
      (* A filename relative to the jail's root *)

type device =
    { dev_name : string;
         (* The UMLMON name, to be used in /shared/disks, with ".dev" 
          * extension.
          *)
      dev_target : string;   
         (* The abs file name of the device in the host system *)
      dev_create_group : int;
         (* The group of the file (only used when created). The owner is
          * always root.
          *)
      dev_create_perm : int;
         (* The permissions of the file *)
      dev_identity : [ `Ignore | `Check | `Update ];
         (* Whether to check/update the identity of the device *)
    }

type hwaddr =
  | MAC of string


type umlglobalspec =
    { umlvardir : string;             (* Directory of the monitor's data directory *)
      umltcp : bool;                  (* Whether to create TCP ports *)
      umltcp_redir : bool;            (* Whether to provide the redir service *)
      umlmasterpw : string option;    (* Master password (MD5) *)
      umlsitecmd : string option;     (* Site command *)
      umlhookcmd : string option;     (* Hook command *)
      umlshared : string;             (* Location of the "shared" directory *)
      umldevs : device list;          (* Device nodes in "shared" *)
      umlhwpool : hwaddr * int;       (* Start address, number of addresses *)
    }

type termdef =
    [ `Pty of string option   (* Pty, optionally logged to a file *)
    ]

type umlterm =
    { termname : string;   (* Name of the terminal from UML's point of view *)
      termdef : termdef;
    }

type umldisk =
    { diskname : string;   (* "ubd" + number *)
      diskdef : jailfile;
      disksync : bool;     (* Whether to force sync mode *)
    }

type hwaddr_spec =
    [ `Unconfigured
    | `HW of hwaddr      (* This MAC address as six bytes *)
    | `HW_from_pool      (* Get MAC address from pool *)
    ]

type net_host_to_host =
    { hth_host_if : string option;       (* Interface name, e.g. eth0 *)
      hth_proxy_if : string;            (* Interface name, e.g. tap0 *)
         (* formerly: hth_guest_if *)
      hth_proxy_addr : string;          (* IP Address, as dotted quad *)
         (* formerly: hth_host_addr *)
      hth_proxy_hwaddr : hwaddr_spec;   (* Optionally set MAC address *)
         (* formerly: hth_host_hwaddr *)
      hth_guest_addr : string;          (* IP Address, as dotted quad *)
      hth_guest_hwaddr : hwaddr_spec;   (* Optionally set MAC address *)
    }

type net_bridged =
    { br_bridge_if : string;             (* Interface name, e.g. br0 *)
      br_proxy_if : string;              (* Interface name, e.g. tap0 *)
      br_proxy_hwaddr : hwaddr_spec;     (* Optionally set MAC address *)
      br_guest_hwaddr : hwaddr_spec;     (* Optionally set MAC address *)
    }

type ethdef =
    [ `Host_to_host of net_host_to_host
    | `Bridged of net_bridged
    | `Unparsed of string
    ]

type umleth =
    { ethname : string;   (* "eth" + number *)
      ethdef : ethdef;
    }

type privileges =
    { priv_uid : int;            (* Effective user *)
      priv_gid : int;            (* Effective default group *)
      priv_groups : int array;   (* Supplementary groups *)
    }


type default_scheduler =
    { sched_def_prio : int option;            (* -19 to 20  *)
      sched_def_affinity : int array option;  (* List of CPU numbers *)
        (* affinity is currently ignored *)
    }

type rr_scheduler =
    { sched_rr_prio : int;                    (* 1 to 99 *)
      sched_rr_affinity : int array option;   (* List of CPU numbers *)
        (* affinity is currently ignored *)
    }

type scheduler =
    [ `Default of default_scheduler
    | `RoundRobin of rr_scheduler
    ]


type umlspec =
    { umlid : string;      (* identifies the UML definition *)
      umlvmuser : string;  (* which user runs the UML *)
      umlvmuserpw : string option;   (* The user password for TCP (MD5) *)
      umlvmgroup : string option; (* The group. Default = main group of user *)
      umlpriv : privileges;  (* Privileges at the time the config is read *)
      umlsched : scheduler;  (* Specifies the scheduler to use *)
      umllogdir : string;  (* Directory where user log files go to *)
      umljaildir : string; (* Directory of the UML jail *)
      umlmem : int;        (* in megs *)
      umlmemlimit : int;   (* in megs *)
      umlkernel : string;  (* kernel, relative to [umldir] *)
      umlkernelargs : string list;  (* kernel arguments. Must not include umid,
				* mem, and the monitored terminals
                                     *)
      umlterms : umlterm list;  (* Monitored consoles and serial lines *)
      umldisks : umldisk list;  (* Defined disks *)
      umlsync : string list;    (* List of sync disks or "all" *)
      umleths : umleth list;    (* Defines eth interfaces *)
      umlmonlog : string option; (* Monitor logfile *)
      umlstart : [ `Boot | `Manual | `Off ]; 
    }


(* File locations:
 * - /etc/umlmon: Global configuration
 * - [umlvardir]/[umlid]/ctrl: Socket connecting to the monitor for this UML instance
 * - [umlvardir]/[umlid]/config: User-specific configuration
 * - [umlvardir]/[umlid]/log: Default location for user logfiles
 * - [umljaildir]/tmp: tmpfs filesystem
 * - [umljaildir]/lock: lock file ???
 * - /var/run/umlmon/[umlid].pid: PID file
 *)

(**********************************************************************)
(* State types                                                        *)
(**********************************************************************)

type ptystate =
    [ `Unconnected of (unit -> unit)
    | `Connected of int32
    ]
  (* `Unconnected: 
   *    The arg function must be called when the state is left, for cleanup 
   * `Connected:
   *    The arg is the connection ID
   *)

class type pty_in_channel =
object
  inherit Uq_engines.async_in_channel
  method notify : unit -> unit
end

class type pty_out_channel =
object
  inherit Uq_engines.async_out_channel
  method notify : unit -> unit
end


type termstatus_pty =
    { ptymaster : Unix.file_descr;
      ptyslavename : string;
      mutable ptylog : Unix.file_descr option;
      ptylogname : string option;
      ptyinch : pty_in_channel;
        (* Reading from this channel reads from the master pty *)
      ptyinch_eng : unit Uq_engines.engine;
        (* The engine connecting ptyinch with the master pty *)
      ptyoutch : pty_out_channel;
        (* Writing to this channel writes to the master pty *)
      ptyoutch_eng : unit Uq_engines.engine;
        (* The engine connecting ptyoutch with the master pty *)
      mutable ptystate : ptystate;
      mutable ptytimer : Unixqueue.group option;
    }

type termstatus = 
    [ `Pty of termstatus_pty
    ]

type umlterm_activation =
    { actv_termname : string;
      actv_termdef : termdef;
      actv_termstatus : termstatus;
      mutable actv_termemu : Termemu.terminal_emulator option;
    }

type umldisk_activation =
    { actv_diskname : string;         (* Name of the UML disk device *)
      actv_cowfile : jailfile option;
      actv_dskfile : jailfile;
    }

type lock_id = Lock_id of int

type umlvm_activation = 
    { mutable actv_vmpid : int option;    (* PID of the VM itself, if known *)
      actv_umlterms : umlterm_activation list;
      actv_monpid : int;              (* PID of the monitor child *)
      actv_monfd : Unix.file_descr;   (* RPC connection with child *)
      actv_monsrv : Rpc_server.t;     (* Server accepting RPC calls of child *)
      actv_lock : lock_id;            (* Locks in the "shared" directory *)
    }

type umlvm =
    [ `Inactive
    | `Stopped of umlvm_activation
    | `Running of umlvm_activation
    | `Starting of umlvm_activation
    | `Finishing  (* Got notification by monitor process, but not yet completely down *)
    ]

type cow_disk_info =
    { cow_backing_file : jailfile;
    }

type disk_access = [ `Invisible | `Read_only | `Read_write ]

type lock_status = [ `None | `Read | `Read_own | `Write | `Write_own ]


type managed_disk_file =
    { md_file_name : jailfile;               (* Relative to jail root *)
      md_file_type : [ `Cow of cow_disk_info | `Dsk | `Dev ];
      md_file_owned : bool;
      (* = Whether the file is exclusively associated to this VM *)
      md_file_access : disk_access;
      md_file_lock : lock_status;
      mutable md_file_status : [ `Exists | `Being_created | `Error of string ]
    }

type umlstatus =
    { umlglobal : umlglobalspec Lazy.t;
      mutable umlspec : umlspec Lazy.t;
      mutable umlvm : umlvm;
      mutable md : managed_disk_file list option;
        (* None: Info not available, must read disks first *)
      mutable ctrlsrv : Rpc_server.t list;
    }

exception Auth_failed of Unix.sockaddr
  (* Raised when client authentication fails. The sockaddr is the peer's address *)

