(* $Id: umlfeed.ml 183 2005-10-13 23:35:44Z gerd $ *)

open Umlmonctrl_aux
open Umlmon_client
open Printf

module Cl = Umlmonctrl_clnt.Ctrl.V1


exception Cannot_connect_term of string


let ues = Unixqueue.create_unix_event_system();;


let tty_setup ?(interrupt=false) ?(crnl=false) f =
  let attr = Unix.tcgetattr Unix.stdin in
  let attr' = 
    { attr with
	Unix.c_ignbrk = not interrupt; (* maybe no CTRL-C = interrupt *)
	c_inlcr = false;      (* No LF -> CR mapping on input *)
	c_icrnl = crnl;       (* Maybe no CR -> LF mapping on input *)
	c_igncr = false;      (* Keep CR *)
	c_istrip = false;     (* 8 bit input *)
	c_ixon = false;       (* No XON/XOFF for input *)
	c_ixoff = false;      (* No XON/XOFF for input *)

	c_opost = false;      (* No output processing *)

	c_isig = interrupt;   (* No keyboard signals *)
	c_icanon = false;     (* No canonical mode *)
	c_echo = false;       (* Disable echo *)
	c_echoe = false;
	c_echok = false;
	c_echonl = false;

	c_vmin = 1;
	c_vtime = 0;
    } in
  Unix.tcsetattr Unix.stdin Unix.TCSANOW attr';
  try
    f();
    Unix.tcsetattr Unix.stdin Unix.TCSANOW attr
  with
      err ->
	Unix.tcsetattr Unix.stdin Unix.TCSANOW attr;
	raise err
;;


let contact id =
  let client = create_client ues id in
  if need_authentication client then (
    prerr_string "Password: ";
    flush stderr;
    tty_setup 
      ~interrupt:true
      ~crnl:true
      (fun () ->
	 let password = read_line() in
	 prerr_endline "\r";
	 flush stderr;
	 if not (authenticate client password) then
	   raise (Rpc.Rpc_server Rpc.Auth_failed);
      );
  );
  client
;;


let do_proc f arg cl =
  match f cl arg with
    | `code_ok -> ()
    | `code_in_progress -> ()
    | `code_error msg ->
	failwith ("Server responds with error: " ^ msg)
    | `code_interrupt ->
	failwith "Interrupt";
;;


let connect cl termid =
  (* Connect terminal *)
  ( match Cl.connect cl termid with
      | `code_ok -> ()
      | _ -> raise (Cannot_connect_term termid)
  );

  print_string "You can disconnect by typing ^]\n";
  flush stdout;

  (* Setup our own tty: *)
  tty_setup
    (fun () ->
       let connected = ref true in
       
       (* Arrange that all data from stdin are sent to the monitor: *)
       let rd_buf = String.create 8192 in

       let cur_in_eng = ref None in

       let rec receive_from_stdin() =
	 let in_eng =
	   new Uq_engines.poll_engine
	     [ Unixqueue.Wait_in Unix.stdin, (-1.0) ]
	     ues in
	 cur_in_eng := Some in_eng;
	 
	 Uq_engines.when_state 
	   ~is_done:(fun _ ->
		       let n = Unix.read Unix.stdin rd_buf 0 (String.length rd_buf) in
		       if n = 0 then raise End_of_file;  (* How can this happen ? *)
		       let s = String.sub rd_buf 0 n in
		       Cl.write'async cl (termid,s,10l) (fun _ -> ());
		       if s = "\029" then (
			 (* Disconnect: *)
			 connected := false;
			 cur_in_eng := None;
			 Cl.disconnect'async cl termid (fun _ -> ());
		       )
		       else
			 receive_from_stdin()
		  )
	   in_eng;
       in
       receive_from_stdin();

       (* Now do everything until you are happy: *)

       Unixqueue.run ues;
    );
  prerr_endline "\nDisconnect.\n";
  Cl.disconnect'async cl termid (fun _ -> ());  (* Just to be sure. *)
;;


let main() =
  let mode = ref `None in
  let umlid = ref None in

  Arg.parse
    (Arg.align
       [ "-connect", 
	Arg.String (fun s -> mode := `Connect s),
	"<con> Connect with console <con> in an interactive session";

       ])
    (fun s ->
       match !umlid with
	 | None -> umlid := Some s
	 | Some _ -> raise(Arg.Bad "Only one VM can be contacted"))
    "usage: umlfeed [options] <umlid>";

  let id =
    match !umlid with
      | None ->
	  failwith "Name of the VM to contact is missing!";
      | Some id ->
	  id
  in

  let client = contact id in
  match !mode with
    | `None        -> failwith "Don't know what to do"
    | `Connect con -> connect client con
;;


try
  Sys.set_signal Sys.sigpipe Sys.Signal_ignore;
  main();
with
  | Failure msg ->
      prerr_endline ("umlwatch: " ^ msg); exit 2
  | Sys_error msg ->
      prerr_endline ("umlwatch: " ^ msg); exit 2
  | Unix.Unix_error(e,_,_) ->
      prerr_endline ("umlwatch: " ^ Unix.error_message e); exit 2
  | Cannot_connect_term id ->
      prerr_endline ("umlwatch: Cannot connect to terminal " ^ id); exit 2
  | Rpc.Rpc_server Rpc.System_err ->
      prerr_endline ("umlwatch: Server replied with 'system error' - look into log files for more information"); exit 2
  | Rpc.Rpc_server Rpc.Auth_failed ->
      prerr_endline ("umlwatch: Authentication failed"); exit 2
  | Rpc.Rpc_server (Rpc.Unavailable_version _) ->
      prerr_endline ("umlwatch: The server does not support this version of the umlmon protocol"); exit 2
  | err ->
      prerr_endline ("umlwatch: Caught exception " ^ Printexc.to_string err); exit 2
;;

